package computations;

import java.util.ArrayList;

import datamodel.Log;
import datamodel.Primitive;
import datamodel.SubSystem;
import datamodel.System;
import datamodel.UpgradeLimitExceededException;

public class Tuner {
	
	public static final boolean debug=false;

	public static final int UPGRADE = 0;

	public static final int REPLICATE = 1;
	
	public static final int PROCESSING = 0;

	public static final int STORAGE = 1;

	public static final int CONNECTIVITY = 2;
	
	public static final int UNIFORM = 0;

	public static final int NONUNIFORM = 1;
	
	public static final String[] methodList={"UPGRADE", "REPLICATE"};
	public static final String[] dimensionList={"PROCESSING", "STORAGE", "CONNECTIVITY"};
	public static final String[] typeList={"UNIFORM", "NONUNIFORM"};	
	
	private System system;
	private Log log;
	
	public Tuner(System system) {	
		this.system=system;
		system.setTuner(this);
		log=new Log();
	}

	public ArrayList<UpgradeLimitExceededException> tuneUniform(Object obj, int dimension, double factor) {
		/*upgrade all hardware under the parent obj. is this correct?*/
		if(!debug){
			writeToLog(Tuner.UNIFORM, dimension, obj, Tuner.UPGRADE, factor);
		}
		ArrayList<UpgradeLimitExceededException> errorList=new ArrayList<UpgradeLimitExceededException>();
		if (obj instanceof SubSystem) {			
			ArrayList<Primitive> primitiveList=System.getAllPrimitives((SubSystem)obj);			
			for (Primitive device : primitiveList) {				
				errorList.add(device.upgrade(dimension, factor));
			}			
		}else if (obj instanceof Primitive) {							
			errorList.add(((Primitive)obj).upgrade(dimension, factor));					
		} 			
		return errorList;
	}

	public Exception tuneNonUniform(int dimension, Object obj, int method, double factor) {		
		if(!debug){
			writeToLog(Tuner.NONUNIFORM, dimension, obj, method, factor);
		}
		if (obj instanceof Primitive) {
			Primitive primitive = (Primitive) obj;
			switch (method) {
			case UPGRADE:				
				return primitive.upgrade(dimension, factor);
			case REPLICATE:				
				return primitive.replicate((int)factor);					
			default:
				return null;
			}			
		} else if (obj instanceof SubSystem) {
			SubSystem ss = (SubSystem) obj;
			switch (method) {
			case UPGRADE:
				return ss.upgradeNonUniform(ss);
			case REPLICATE:				
				return ss.replicate((int)factor);
			default:				
				return null;
			}			
		}else{
			return new Exception("Nothing to tune!");
		}
	}
	
	private void writeToLog(int type, int dimension, Object obj, int method,
			double factor) {
		String logAction = new String(typeList[type] + ":" + obj.toString() + ":"
				+ methodList[method] + ":" + dimensionList[dimension] + ":"
				+ factor);
		log.addLogPost(system.getRootNode(), logAction);
	}
	
}
