package datamodel;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.NumberFormat;
import java.util.Vector;

import computations.Tuner;

public class DataBaseHandler {

	public DataBaseHandler(){
		
	}
	
	public static double getNextUpgradeFactor(double capacity, double desiredUpgradeFactor, int dimension, int deviceType){
		if (capacity==0) {
			return -1;
		}
		double f=-1;
		double upgradedCapacity=capacity * desiredUpgradeFactor;					
		switch (dimension) {
		case Tuner.PROCESSING:
			//the speed is upgraded
			double closestSpeed=getClosestSpeed(upgradedCapacity, deviceType);		
			if(closestSpeed==-1){
				return -1;
			}
			f=closestSpeed/capacity;
			break;
		case Tuner.STORAGE:
			//the storage is upgraded
			double closestStorage=getClosestStorage(upgradedCapacity, deviceType);	
			if(closestStorage==-1){
				return -1;
			}
			f=closestStorage/capacity;			
			break;
		case Tuner.CONNECTIVITY:
			//the connectivity is upgraded
			double closestConnectivity=getClosestConnectivity(upgradedCapacity, deviceType);	
			if(closestConnectivity==-1){
				return -1;
			}
			f=closestConnectivity/capacity;
			break;
		default:
			break;
		}		
		NumberFormat form=NumberFormat.getInstance();
		form.setGroupingUsed(false);
		f=new Double(form.format(f)).doubleValue();
		return f;
	}
	private static Connection newConnection() throws SQLException{
		try {
			Class.forName("org.gjt.mm.mysql.Driver").newInstance();			
		} catch (Exception e) {			
			java.lang.System.out.println("Error with DB driver");
			e.printStackTrace();
			java.lang.System.exit(0);
		}		
		String url="jdbc:mysql://mysql.stud.ntnu.no/anderh_master";
		String username="anderh_master";
		String password="1234";
		Connection con = DriverManager.getConnection(url, username, password);		
		return con;
	}
	
	private static double getClosestSpeed(double desiredSpeed, int deviceType){
		/*connect to db and return 1st row where db.speed >= speed*/		
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT speed "+
					"FROM devices " +
					"WHERE speed>="+desiredSpeed+" "+
					"AND type="+deviceType+" "+
					"ORDER BY speed "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return -1;
			}else{	
				double s=result.getDouble("speed");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return -1;
		}		
	}
	private static double getClosestStorage(double desiredStorage, int deviceType){
		/*connect to db and return 1st row where db.storage >= desiredstorage*/
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT storage "+
					"FROM devices " +
					"WHERE storage>="+desiredStorage+" "+
					"AND type="+deviceType+" "+
					"ORDER BY storage "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return -1;
			}else{	
				double s=result.getDouble("storage");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return -1;
		}		
	}
	private static double getClosestConnectivity(double desiredConnectivity, int deviceType){
		/*connect to db and return 1st row where db.connec >= desiredconnec*/
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT connectivity "+
					"FROM devices " +
					"WHERE connectivity>="+desiredConnectivity+" "+
					"AND type="+deviceType+" "+
					"ORDER BY connectivity "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return -1;
			}else{	
				double s=result.getDouble("connectivity");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return -1;
		}		
	}

	public static double getMaxUpgradeFactor(double capacity, int dimension, int deviceType){
		double f=-1;				
		if (capacity==0) {
			return 1;
		}
		switch (dimension) {
		case Tuner.PROCESSING:
			double maxSpeed=getMaxSpeed(deviceType);			
			f=maxSpeed/capacity;
			break;
		case Tuner.STORAGE:
			double maxStorage=getMaxStorage(deviceType);			
			f=maxStorage/capacity;			
			break;
		case Tuner.CONNECTIVITY:
			double maxConnectivity=getMaxConnectivity(deviceType);			
			f=maxConnectivity/capacity;
			break;
		default:
			break;
		}		
		NumberFormat form=NumberFormat.getInstance();
		form.setGroupingUsed(false);
		f=new Double(form.format(f)).doubleValue();
		return f;
	}
	
	private static double getMaxSpeed(int deviceType){
		/*connect to db and return 1st row where db.speed >= speed*/		
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT speed "+
					"FROM devices " +			
					"WHERE type="+deviceType+" "+
					"ORDER BY speed DESC "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return 0;
			}else{	
				double s=result.getDouble("speed");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return 0;
		}		
	}
	private static double getMaxStorage(int deviceType){
		/*connect to db and return 1st row where db.speed >= speed*/		
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT storage "+
					"FROM devices " +				
					"WHERE type="+deviceType+" "+	
					"ORDER BY storage DESC "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return 0;
			}else{	
				double s=result.getDouble("storage");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return 0;
		}		
	}
	private static double getMaxConnectivity(int deviceType){
		/*connect to db and return 1st row where db.speed >= speed*/		
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT connectivity "+
					"FROM devices " +					
					"WHERE type="+deviceType+" "+
					"ORDER BY connectivity DESC "+
					"LIMIT 1;"); 
			if(!result.next()){//no device is good enough
				stmt.close();
				con.close();
				return 0;
			}else{	
				double s=result.getDouble("connectivity");	
				stmt.close();
				con.close();
				return s;
			}
		} catch (SQLException e) {
			return 0;
		}		
	}

	public static Vector<String> getAllDeviceDescriptions(){
		/*connect to db and return all descriptions*/		
		try {			
			Connection con = newConnection();
			if(con==null){
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery(
					"SELECT description "+
					"FROM devices " +					
					"ORDER BY id;");
			if(!result.next()){//no devices exist
				stmt.close();
				con.close();
				return null;
			}else{	
				Vector<String> devices=new Vector<String>();
				do {
					devices.add(result.getString("description").trim());
				} while (result.next());				
				stmt.close();
				con.close();
				return devices;
			}
		} catch (SQLException e) {
			return null;
		}		
	}
	
	public static double[] getDeviceData(int index){
		/*connect to db and return all fields for the device with id=index*/		
		try {			
			Connection con = newConnection();		
			if(con==null){				
				throw new SQLException();
			}
			Statement stmt=con.createStatement();
			ResultSet result = stmt.executeQuery("SELECT * FROM devices WHERE id="+index+";");
			if(!result.next()){//id is wrong				
				stmt.close();
				con.close();
				return null;
			}else{					
				double[] data=new double[5];//four columns are interesting						
				data[0]=result.getDouble("speed");	
				data[1]=result.getDouble("storage");
				data[2]=result.getDouble("connectivity");
				data[3]=(double)result.getInt("max_replication");	
				data[4]=(double)result.getInt("type");
				stmt.close();
				con.close();				
				return data;
			}
		} catch (SQLException e) {		
			return null;
		}		
	}
}
