package datamodel;

import java.util.ArrayList;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import computations.Tuner;

public class Primitive extends DefaultMutableTreeNode implements Component {

	static final long serialVersionUID = 1;

	private String name;

	private int deviceType=0;

	private int deviceDBID=0;

	private final long typeCode;

	/* Original and scaled service demands */
	private double originalSD=1;

	private double scaleSD=1;

	private double devolvedWork=1;

	private double processingSpeed;

	private double storageCapacity;

	private double connectivity;

	private double upgradeFactorProcessing = 1;

	private double upgradeFactorStorage = 1;

	private double upgradeFactorConnectivity = 1;

	private int replication = 1;

	private int maxReplication = 9999;

	public Primitive() {
		// create unique idnr
		long l1 = java.lang.System.currentTimeMillis();
		long l2;
		for (; (l2 = java.lang.System.currentTimeMillis()) == l1;)
			;
		typeCode = l2;
	}

	public Primitive(String name) {
		this.name = name;
		long l1 = java.lang.System.currentTimeMillis();
		long l2;
		for (; (l2 = java.lang.System.currentTimeMillis()) == l1;);
		typeCode = l2;
	}
	public Primitive(String name, double devolvedWork, int replications) {
		this.name = name;
		this.devolvedWork=devolvedWork;
		this.replication=replications;
		long l1 = java.lang.System.currentTimeMillis();
		long l2;
		for (; (l2 = java.lang.System.currentTimeMillis()) == l1;);
		typeCode = l2;
	}

	public Primitive(String name, SubSystem parent) {
		this.name = name;
		setParent(parent);
		long l1 = java.lang.System.currentTimeMillis();
		long l2;
		for (; (l2 = java.lang.System.currentTimeMillis()) == l1;)
			;
		typeCode = l2;
	}

	public double getDevolvedWork() {
		return devolvedWork;
	}

	public void setDevolvedWork(double devolvedWork) {
		this.devolvedWork = devolvedWork;
	}

	public void setProcessingSpeed(double processingSpeed) {
		this.processingSpeed = processingSpeed;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getReplication() {
		return replication;
	}
	public void setReplication(int replication){
		this.replication=replication;
	}
	
	public double getScaleSD() {
		return scaleSD;
	}

	public double getUpgradeFactorProcessing() {
		return upgradeFactorProcessing;
	}

	public UpgradeLimitExceededException setUpgradeFactorProcessing(
			double upgradeFactorProcessing) {
		double f = DataBaseHandler.getNextUpgradeFactor(processingSpeed,
				upgradeFactorProcessing, Tuner.PROCESSING, deviceType);
		if (f == -1) {
			return new UpgradeLimitExceededException(getName(),
					upgradeFactorProcessing, Tuner.PROCESSING);
		} else {
			this.upgradeFactorProcessing = f;
			return null;
		}
	}

	public double getUpgradeFactorConnectivity() {
		return upgradeFactorConnectivity;
	}

	public UpgradeLimitExceededException setUpgradeFactorConnectivity(
			double upgradeFactorConnectivity) {
		double f = DataBaseHandler.getNextUpgradeFactor(connectivity,
				upgradeFactorConnectivity, Tuner.CONNECTIVITY, deviceType);
		if (f == -1) {
			return new UpgradeLimitExceededException(getName(),
					upgradeFactorConnectivity, Tuner.CONNECTIVITY);
		} else {
			this.upgradeFactorConnectivity = f;
			return null;
		}
	}

	public double getUpgradeFactorStorage() {
		return upgradeFactorStorage;
	}

	public UpgradeLimitExceededException setUpgradeFactorStorage(
			double upgradeFactorStorage) {
		double f = DataBaseHandler.getNextUpgradeFactor(storageCapacity,
				upgradeFactorStorage, Tuner.STORAGE, deviceType);
		if (f == -1) {
			return new UpgradeLimitExceededException(getName(),
					upgradeFactorStorage, Tuner.STORAGE);
		} else {
			this.upgradeFactorStorage = f;
			return null;
		}
	}

	public String toString() {
		return this.getName();
	}

	public Primitive shallowCopy() {
		return (Primitive) super.clone();
	}

	public UpgradeLimitExceededException upgrade(int dimension, double factor) {
		switch (dimension) {
		case Tuner.PROCESSING:
			return setUpgradeFactorProcessing(factor);
		case Tuner.STORAGE:
			return setUpgradeFactorStorage(factor);
		case Tuner.CONNECTIVITY:
			return setUpgradeFactorConnectivity(factor);
		default:
			return null;
		}
	}

	public ReplicationLimitExceededException replicate(int factor) {
		if (factor <= maxReplication) {
			this.replication = factor;
			return null;
		} else {
			return new ReplicationLimitExceededException(getName(), replication);
		}		
	}

	public void calcScaleSD() {
		scaleSD=(originalSD / upgradeFactorProcessing);
	}

	public ArrayList<UpgradeLimitExceededException> init(double reqProc,
			double reqStor, double reqConn) {
		ArrayList<UpgradeLimitExceededException> errorList = new ArrayList<UpgradeLimitExceededException>();
		errorList.add(setUpgradeFactorProcessing(reqProc));
		errorList.add(setUpgradeFactorStorage(reqStor));
		errorList.add(setUpgradeFactorConnectivity(reqConn));
		return errorList;
	}

	public double getConnectivity() {
		return connectivity;
	}

	public void setConnectivity(double connectivity) {
		this.connectivity = connectivity;
	}

	public int getMaxReplication() {
		return maxReplication;
	}

	public void setMaxReplication(int maxReplication) {
		this.maxReplication = maxReplication;
	}

	public double getProcessingSpeed() {
		return processingSpeed;
	}

	public double getStorageCapacity() {
		return storageCapacity;
	}

	public void setStorageCapacity(double storageCapacity) {
		this.storageCapacity = storageCapacity;
	}

	public int getDeviceDBID() {
		return deviceDBID;
	}

	public void setDeviceDBID(int deviceDBID) {
		this.deviceDBID = deviceDBID;
	}

	public void makeTree(DefaultTreeModel model) {
		model.insertNodeInto(this, parent, parent.getChildCount());
	}

	public Component clone(Component parent) {
		Primitive newClone = this.shallowCopy();//primitives do not have children		
		newClone.setName(this.name+parent.getIndex(parent)+"");
		return newClone;
	}

	public long getTypeCode() {
		return typeCode;
	}

	public void calculateOriginalSD() {
		originalSD = devolvedWork / processingSpeed;
	}

	public void setData(){
		double[] data=DataBaseHandler.getDeviceData(deviceDBID+1);
		this.setProcessingSpeed(data[0]);
		this.setStorageCapacity(data[1]);
		this.setConnectivity(data[2]);
		this.setMaxReplication((int)(data[3]));	
	}

	public int getDeviceType() {
		return deviceType;
	}

	public void setDeviceType(int deviceType) {
		this.deviceType = deviceType;
	}
}
