package datamodel;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

public class SubSystem extends DefaultMutableTreeNode implements Component{
	
	static final long serialVersionUID=1;
	
	private final long typeCode;
	
	private String name;

	private int maxReplication=1;

	public SubSystem() {
		long l1=java.lang.System.currentTimeMillis();
		long l2;
		for(;(l2=java.lang.System.currentTimeMillis())==l1;);
		typeCode=l2;
	}
	public SubSystem(String name) {
		this.name=name;
		long l1=java.lang.System.currentTimeMillis();
		long l2;
		for(;(l2=java.lang.System.currentTimeMillis())==l1;);
		typeCode=l2;
	}
	public SubSystem(String name, int maxReplication) {
		this.name=name;
		this.maxReplication=maxReplication;
		long l1=java.lang.System.currentTimeMillis();
		long l2;
		for(;(l2=java.lang.System.currentTimeMillis())==l1;);
		typeCode=l2;
	}
	public SubSystem(String name, SubSystem parent) {
		this.name=name;
		setParent(parent);
		long l1=java.lang.System.currentTimeMillis();
		long l2;
		for(;(l2=java.lang.System.currentTimeMillis())==l1;);
		typeCode=l2;
	}

	public String toString(){
		return this.getName();
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
			
	public Component clone(Component parentComponent){	
		SubSystem newClone = this.shallowCopy();				
		Enumeration e=this.children();
		while(e.hasMoreElements()){
			newClone.insert(((Component)e.nextElement()).clone(newClone), newClone.getChildCount());
		}		
		return newClone;		
	}

	private SubSystem shallowCopy() {
		SubSystem tmp = (SubSystem) super.clone();
		tmp.setName(name+getCurrentReplicationsCount()+"");
		return tmp;
	}

	public void makeTree(DefaultTreeModel model) {		
		model.insertNodeInto(this, parent, parent.getChildCount());
		Enumeration e=this.children();
		while(e.hasMoreElements()){
			((Component)e.nextElement()).makeTree(model);			
		}
	}
	
	public int getCurrentReplicationsCount(){
		int currentReplications=-1;		
		SubSystem parent=(SubSystem)this.parent;
		if(parent==null){//root node
			return 1;
		}		
		for (int i=0;i<parent.getChildCount();i++) {	
			if (parent.getChildAt(i) instanceof SubSystem) {
				SubSystem child = (SubSystem) parent.getChildAt(i);
				if(child.typeCode==this.typeCode){
					currentReplications++;
				}
			}
			
		}
		return currentReplications;
	}
	
	public ReplicationLimitExceededException replicate(int factor) {		
		if((factor+getCurrentReplicationsCount()) <= maxReplication){
			/* just clone i guess
			 */			
			SubSystem copy;
			for (int i = 1; i < factor+1; i++) {
				copy=(SubSystem)clone((Component)this.parent);
				copy.setName(getName()+i+"");				
				this.parent.insert(copy, this.parent.getIndex(this)+i);				
			}			
			return null;
		}else {
			//notify user
			return new ReplicationLimitExceededException(getName(),factor);
		}
	}
	public UpgradeLimitExceededException upgradeNonUniform(SubSystem ss){
		//show subsystems for the subsystem?		
		return null;
	}
	public int getMaxReplication() {
		return maxReplication;
	}
	public void setMaxReplication(int maxReplication) {
		this.maxReplication = maxReplication;
	}
	public long getTypeCode() {
		return typeCode;
	}
	
}