package datamodel;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import computations.Tuner;

public class System {

	private SubSystem rootNode;

	private double OP;

	private int opType;
	
	private int baselineLoad=0;
	
	private double requirementProcessing;
	private double requirementStorage;
	private double requirementConnectivity;
		
	private Tuner tuner;

	public System() {
		tuner=new Tuner(this);
	}

	public static ArrayList<Primitive> getAllPrimitives(SubSystem parent) {
		ArrayList<Primitive> hWList = new ArrayList<Primitive>();
		recursiveGet(parent, hWList);
		return hWList;
	}

	private static void recursiveGet(Component node, ArrayList<Primitive> dev) {
		if (node instanceof Primitive) {
			Primitive device = (Primitive) node;
			dev.add(device);
		}else{
			Enumeration e=node.children();
			while(e.hasMoreElements()){
				recursiveGet((Component)e.nextElement(), dev);
			}		
		}
	}

	public double getOP() {
		return OP;
	}

	public void setOP(double op) {
		OP = op;
	}

	public SubSystem getRootNode() {
		return rootNode;
	}

	public void setRootNode(SubSystem topNode) {
		this.rootNode = topNode;
	}

	public int getOpType() {
		return opType;
	}

	public void setOpType(int opType) {
		this.opType = opType;
	}

	public int getBaselineLoad() {
		return baselineLoad;
	}

	public void setBaselineLoad(int baselineLoad) {
		this.baselineLoad = baselineLoad;
	}

	public ArrayList<UpgradeLimitExceededException> initSearch() {
		
		ArrayList<UpgradeLimitExceededException> errorList=new ArrayList<UpgradeLimitExceededException>();
		errorList.addAll(tuner.tuneUniform(rootNode, Tuner.PROCESSING, requirementProcessing));
		errorList.addAll(tuner.tuneUniform(rootNode, Tuner.STORAGE, requirementStorage));
		errorList.addAll(tuner.tuneUniform(rootNode, Tuner.CONNECTIVITY, requirementConnectivity));
		return errorList;
		/*
		ArrayList<UpgradeLimitExceededException> errorList=new ArrayList<UpgradeLimitExceededException>();
		for (Primitive p : getAllPrimitives(rootNode)) {
			errorList.addAll(p.init(requirementProcessing, requirementStorage, requirementConnectivity));
		}
		return errorList;
		*/
	}
	public void setTuner(Tuner tuner){
		this.tuner=tuner;
	}
	
	public void recoverSystemState(LogPost chosenLogPost){
		rootNode=chosenLogPost.getSystemStateRootNode();
	}
	
	public void saveSystem(SubSystem root){
		//dump to file on disk
	}

	public double getRequirementConnectivity() {
		return requirementConnectivity;
	}

	public void setRequirementConnectivity(double requirementConnectivity) {
		this.requirementConnectivity = requirementConnectivity;
	}

	public double getRequirementProcessing() {
		return requirementProcessing;
	}

	public void setRequirementProcessing(double requirementProcessing) {
		this.requirementProcessing = requirementProcessing;
	}

	public double getRequirementStorage() {
		return requirementStorage;
	}

	public void setRequirementStorage(double requirementStorage) {
		this.requirementStorage = requirementStorage;
	}

	public Tuner getTuner() {
		return tuner;
	}
}
