package gui.setup;


import gui.tune.TunePanel;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import computations.GainCalculator;

import datamodel.System;
import datamodel.UpgradeLimitExceededException;

public class BaselinePanel extends JPanel implements ActionListener{
	
	private BaselineCalcPanel baseCalcPanel;
	
	private JTextField loadText;
	
	private JButton initButton;	
	
	private JFrame frame;
	private System system;
	private GainCalculator gainCalc;
	
	public BaselinePanel(JFrame frame, System system) {
		super();
		this.frame=frame;
		this.system=system;
		gainCalc=new GainCalculator(system);
		baseCalcPanel=new BaselineCalcPanel();
		baseCalcPanel.getCalcButton().addActionListener(this);
		baseCalcPanel.getCancelButton().addActionListener(this);
		baseCalcPanel.getOptionsButton().addActionListener(this);
				
		JLabel loadLabel=new JLabel("Baseline load:");
		loadText=new JTextField();
		loadText.setEditable(false);
				
		JPanel loadPanel=new JPanel();
		loadPanel.setLayout(new GridLayout(0,2));
		loadPanel.add(loadLabel);
		loadPanel.add(loadText);			
		loadPanel.setBorder(BorderFactory.createTitledBorder("Baseline load"));
		
		JPanel basePanel=new JPanel(new BorderLayout());
		basePanel.add(baseCalcPanel, BorderLayout.CENTER);
		basePanel.add(loadPanel, BorderLayout.SOUTH);
			
		JPanel okPanel=new JPanel();
		initButton=new JButton("Initialise search and tune");
		initButton.addActionListener(this);
		initButton.setEnabled(false);
		okPanel.add(initButton);
		
		setLayout(new BorderLayout());
		add(basePanel, BorderLayout.CENTER);
		add(okPanel, BorderLayout.SOUTH);
		
	}
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==baseCalcPanel.getCalcButton()){
			//calculate baseline load			
			gainCalc.setCalcMethod(baseCalcPanel.getCalculationMethod());
			system.setBaselineLoad(gainCalc.calculateLoadAtOP());
			loadText.setText(system.getBaselineLoad()+"");		
			initButton.setEnabled(true);
		}else if(e.getSource()==baseCalcPanel.getOptionsButton()){
			//open calcoptions panel in jdialog
			JOptionPane.showMessageDialog(this, "No options yet..", "Sorry", JOptionPane.INFORMATION_MESSAGE);
		}else if(e.getSource()==initButton && system.getBaselineLoad()>0){
			//set baseline data in system, and open tunepanel in frame			
			ArrayList<UpgradeLimitExceededException> errorList=system.initSearch();
			boolean justErrors=true;
			while(errorList.remove(null))justErrors=false;//remove null objects
			if(errorList.size()>0){//give feedback if errors occur
				JTextArea text=new JTextArea();
				for (UpgradeLimitExceededException exception : errorList) {
					text.append(exception.getMessage()+"\n");
				}
				JOptionPane.showMessageDialog(this, text, "Upgrade error", JOptionPane.ERROR_MESSAGE);				
			}			
			if(!justErrors){
				gainCalc.calculateGainProcessing();
				gainCalc.calculateGainStorage();			
				gainCalc.calculateGainConnectivity();	
			}			
			frame.getContentPane().removeAll();
			frame.getContentPane().add(new TunePanel(frame, system, gainCalc));
			frame.pack();
			frame.setVisible(true);
		}else if(e.getSource()==baseCalcPanel.getCancelButton()){
			//exit program
			frame.dispose();
		}		
	}
}
