package gui.setup;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import datamodel.DataBaseHandler;
import datamodel.Primitive;

public class PrimitivePropertiesPanel extends JPanel implements ActionListener {

	private JTextField nameText;
	
	private JComboBox deviceCombo;
	
	private JTextField speedText;
	private JTextField storageText;
	private JTextField connectivityText;
	
	private JTextField workText;
	private JTextField replicationText;
	private JTextField maxReplicationText;
	
	private JButton okButton;
	private JButton cancelButton;
	
	private Primitive device;
	private int deviceType=0;
	
	private JDialog frame;

	public PrimitivePropertiesPanel(JDialog frame, Primitive device) {		
		super(new BorderLayout());
		this.device=device;
		this.frame=frame;
		JLabel nameLabel=new JLabel("Name:");
		nameText=new JTextField();
		JLabel deviceLabel=new JLabel("Device type:");
		deviceCombo=new JComboBox(DataBaseHandler.getAllDeviceDescriptions());	
		deviceCombo.addActionListener(this);
		JLabel speedLabel=new JLabel("Speed:");
		speedText=new JTextField();
		speedText.setEditable(false);
		JLabel storageLabel=new JLabel("Storage:");
		storageText=new JTextField();
		storageText.setEditable(false);
		JLabel connectLabel=new JLabel("Connectivity:");
		connectivityText=new JTextField();
		connectivityText.setEditable(false);
		JLabel maxReplicationLabel=new JLabel("Maximum replication:");
		maxReplicationText=new JTextField();
		maxReplicationText.setEditable(false);
		JLabel workLabel=new JLabel("Devolved work:");
		workText=new JTextField();
		JLabel replicationLabel=new JLabel("Replication:");
		replicationText=new JTextField();
		
		JPanel devicePanel=new JPanel(new GridLayout(4,2));		
		devicePanel.add(nameLabel);
		devicePanel.add(nameText);	
		devicePanel.add(workLabel);
		devicePanel.add(workText);
		devicePanel.add(replicationLabel);
		devicePanel.add(replicationText);
		devicePanel.add(deviceLabel);
		devicePanel.add(deviceCombo);	
		
		JPanel mainPanel=new JPanel(new GridLayout(4,2));		
		mainPanel.add(speedLabel);
		mainPanel.add(speedText);	
		mainPanel.add(storageLabel);
		mainPanel.add(storageText);	
		mainPanel.add(connectLabel);
		mainPanel.add(connectivityText);	
		mainPanel.add(maxReplicationLabel);
		mainPanel.add(maxReplicationText);
			
				
		//mainPanel.add(Box.createRigidArea(new Dimension(10,0)));
		//mainPanel.add(Box.createRigidArea(new Dimension(10,0)));
		okButton=new JButton("OK");
		okButton.addActionListener(this);
		cancelButton=new JButton("Cancel");
		cancelButton.addActionListener(this);
		JPanel okPanel=new JPanel();
		okPanel.add(okButton);
		okPanel.add(cancelButton);
		
		setDeviceData(deviceCombo.getSelectedIndex());
				
		add(devicePanel, BorderLayout.WEST);
		add(mainPanel, BorderLayout.EAST);		
		add(okPanel, BorderLayout.SOUTH);
		
		if(device.getName()!=null){
			initData();
		}
		frame.getRootPane().setDefaultButton(okButton);
	}
	
	private void initData(){
		nameText.setText(device.getName());
		workText.setText(device.getDevolvedWork()+"");
		replicationText.setText(device.getReplication()+"");
		deviceCombo.setSelectedIndex(device.getDeviceDBID());
	}
	
	private void setDeviceData(int index){
		double[] data=DataBaseHandler.getDeviceData(index+1);
		speedText.setText(data[0]+"");
		storageText.setText(data[1]+"");
		connectivityText.setText(data[2]+"");
		maxReplicationText.setText((int)data[3]+"");
		deviceType=(int)data[4];
	}
	public void actionPerformed(ActionEvent e) {
		// listen to okbutton
		if (e.getSource()==deviceCombo) {
			setDeviceData(deviceCombo.getSelectedIndex());			
		} else if (e.getSource()==okButton) {						
			try{			
				if(nameText.getText()==null){
					throw new NullPointerException("No name! ");
				}
				int rep=Integer.parseInt(replicationText.getText());
				int maxRep=(int)(Double.parseDouble(maxReplicationText.getText()));
				if(rep>maxRep){
					throw new Exception("Replication > MaxReplication! ");
				}
				device.setName(nameText.getText());
				device.setDeviceDBID(deviceCombo.getSelectedIndex());
				device.setDevolvedWork(Double.parseDouble(workText.getText()));
				device.setReplication(rep);
				device.setProcessingSpeed(Double.parseDouble(speedText.getText()));
				device.setStorageCapacity(Double.parseDouble(storageText.getText()));
				device.setConnectivity(Double.parseDouble(connectivityText.getText()));
				device.setMaxReplication(maxRep);		
				device.setDeviceType(deviceType);
				frame.dispose();				
			}catch (Exception x) {
				JOptionPane.showMessageDialog(this, "Fill inn all the fields correctly!", 
						"Missing data", JOptionPane.ERROR_MESSAGE);	
			}			
		}else if (e.getSource()==cancelButton) {
			frame.dispose();
		}
		
	}
}
