package gui.setup;

import gui.SAMeTreePanel;
import io.SAMeFileReader;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import datamodel.Primitive;
import datamodel.SubSystem;
import datamodel.System;

public class SystemSetupPanel extends JPanel implements TreeSelectionListener, ActionListener{
	
	private SAMeTreePanel treePanel;
	
	private JButton subsysButton;
	private JButton primitiveButton;
	private JButton propertyButton;
	private JButton delButton;
	private JButton readFileButton;	
	private JButton okButton;

	private JFrame frame;
	private System system;
	
	final JFileChooser fc;
	
	private DefaultMutableTreeNode markedNode=null;
	
	public SystemSetupPanel(JFrame frame, System system) {
		super();
		this.frame=frame;
		this.system=system;		
		fc= new JFileChooser();
		subsysButton=new JButton("New Subsystem");
		subsysButton.addActionListener(this);
		primitiveButton=new JButton("New Primitive");
		primitiveButton.addActionListener(this);
		propertyButton=new JButton("Properties");
		propertyButton.addActionListener(this);
		delButton=new JButton("Delete");
		delButton.addActionListener(this);
		readFileButton=new JButton("Read SP model from file");
		readFileButton.addActionListener(this);
		okButton=new JButton("Proceed");
		okButton.addActionListener(this);
		
		JPanel buttonPanel=new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));
		buttonPanel.add(Box.createRigidArea(new Dimension(0,20)));
		buttonPanel.add(subsysButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0,10)));
		buttonPanel.add(primitiveButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0,10)));
		buttonPanel.add(propertyButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0,10)));
		buttonPanel.add(delButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0,30)));
		buttonPanel.add(readFileButton);
		buttonPanel.add(Box.createRigidArea(new Dimension(0,100)));
		buttonPanel.add(okButton);
		
		treePanel=new SAMeTreePanel();			
		treePanel.getTree().setModel(treePanel.makeTreeModel(system.getRootNode()));
		treePanel.getTree().addTreeSelectionListener(this);		
		markedNode=system.getRootNode();
		
		setLayout(new BorderLayout());
		add(treePanel, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.EAST);		
		setOpaque(true);
		setPreferredSize(new Dimension(800,600));//suits most screens
	}
	public void valueChanged(TreeSelectionEvent arg0) {
		//listen to tree and change markedNode
		if (treePanel.getTree().getLastSelectedPathComponent() instanceof SubSystem) {
			markedNode = (SubSystem) treePanel.getTree().getLastSelectedPathComponent();
			
		}else if (treePanel.getTree().getLastSelectedPathComponent() instanceof Primitive) {
			markedNode = (Primitive) treePanel.getTree().getLastSelectedPathComponent();			
		}	
	}
	public void actionPerformed(ActionEvent e) {
		// listen to buttons
		if(e.getSource()==subsysButton){
			if(markedNode instanceof SubSystem){				
				//only subsystems, not primitives, can have children
				SubSystem newChildSs=new SubSystem();
				JDialog win=new JDialog(frame, "New subsystem", true);
				win.getContentPane().add(new SubSysPropertiesPanel(win, newChildSs));
				win.pack();
				win.setVisible(true);				
				//add new child to parent, which is markedNode. name cannot be null!
				if(newChildSs.getName()!=null){
					((DefaultTreeModel)treePanel.getTree().getModel()).insertNodeInto(
							newChildSs, markedNode, markedNode.getChildCount());		
					treePanel.getTree().scrollPathToVisible(new TreePath(newChildSs.getPath()));
				}
			}else {
				JOptionPane.showMessageDialog(this, 
						"Primitives cannot have children", "Not allowed", JOptionPane.ERROR_MESSAGE);
			}
		}else if (e.getSource()==primitiveButton) {
			if(markedNode instanceof SubSystem){				
				//only subsystems, not primitives, can have children
				Primitive newChildPrimitive=new Primitive();
				JDialog win=new JDialog(frame, "New subsystem", true);
				win.getContentPane().add(new PrimitivePropertiesPanel(win, newChildPrimitive));
				win.pack();
				win.setVisible(true);				
				//add new child to parent, which is markedNode. name cannot be null!
				if(newChildPrimitive.getName()!=null){
					((DefaultTreeModel)treePanel.getTree().getModel()).insertNodeInto(
							newChildPrimitive, markedNode, markedNode.getChildCount());		
					treePanel.getTree().scrollPathToVisible(new TreePath(newChildPrimitive.getPath()));
				}
			}else {
				JOptionPane.showMessageDialog(this, 
						"Primitives cannot have children", "Not allowed", JOptionPane.ERROR_MESSAGE);
			}
		}else if (e.getSource()==propertyButton) {
			if(markedNode instanceof SubSystem){	
				JDialog win=new JDialog(frame, "Subsystem properties", true);
				win.getContentPane().add(new SubSysPropertiesPanel(win, (SubSystem)markedNode));
				win.pack();
				win.setVisible(true);				
			}else {
				JDialog win=new JDialog(frame, "Primitive properties", true);
				win.getContentPane().add(new PrimitivePropertiesPanel(win, (Primitive)markedNode));
				win.pack();
				win.setVisible(true);
			}
			treePanel.getTree().setModel((DefaultTreeModel)treePanel.getTree().getModel());
			treePanel.getTree().scrollPathToVisible(new TreePath(markedNode.getPath()));
		}else if (e.getSource()==delButton) {
			int answer=JOptionPane.showConfirmDialog(
					frame, "Are you sure you want to delete this?", "Confirm delete", JOptionPane.YES_NO_OPTION);
			if(answer==JOptionPane.YES_OPTION){
				DefaultMutableTreeNode newMarked=(DefaultMutableTreeNode)markedNode.getParent();
				((DefaultTreeModel)treePanel.getTree().getModel()).removeNodeFromParent(markedNode);	
				markedNode=newMarked;		
			}
		} else if (e.getSource() == readFileButton) {
			try {
				int returnVal = fc.showOpenDialog(this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					// This is where we open the file.
					SubSystem newRoot = SAMeFileReader.readSPModel(file);
					system.setRootNode(newRoot);
					treePanel.getTree().setModel(
							treePanel.makeTreeModel(newRoot));
					markedNode = system.getRootNode();
				} else {
					return;
				}
			} catch (Exception e1) {
				JOptionPane.showMessageDialog(this, "File format error!",
						"Input exception", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}else if (e.getSource()==okButton) {
			frame.getContentPane().removeAll();
			frame.getContentPane().add(new RequirementPanel(frame, system));	
			frame.pack();
		}else{
			return;
		}
	}
}
