package gui.tune;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import computations.GainCalculator;
import computations.Tuner;

import datamodel.Component;
import datamodel.Primitive;
import datamodel.SubSystem;

public class ReplicationPanel extends JPanel implements ActionListener{
	
	private GainCalcPanel gainCalcPanel;
	
	private JTextField replicationText;
	
	private Window frame;
	private Tuner tuner;
	private GainCalculator gainCalc;
	private Component markedNode;
	private int dimension;

	public ReplicationPanel(Window frame, Tuner tuner, GainCalculator gainCalc, 
			Component markedNode, int dimension) {
		super(new BorderLayout());
		this.frame=frame;
		this.tuner=tuner;
		this.gainCalc=gainCalc;
		this.markedNode=markedNode;
		this.dimension=dimension;
				
		JLabel replicationLabel=new JLabel("Replication factor:");
		replicationText=new JTextField();
		if(markedNode instanceof Primitive){
			replicationText.setText(((Primitive)markedNode).getReplication()+"");
			replicationText.selectAll();
		}
		
		JPanel textPanel=new JPanel(new GridLayout(0,2));
		textPanel.add(replicationLabel);
		textPanel.add(replicationText);
		
		gainCalcPanel=new GainCalcPanel();
		
		gainCalcPanel.getCalcButton().addActionListener(this);
		gainCalcPanel.getOptionsButton().addActionListener(this);
		gainCalcPanel.getCancelButton().addActionListener(this);
		
		add(textPanel, BorderLayout.NORTH);
		add(gainCalcPanel, BorderLayout.SOUTH);
	}

	public void actionPerformed(ActionEvent e) {				
		int factor = 1;
		if (e.getSource() == gainCalcPanel.getCalcButton()) {
			try {
				factor = Integer.parseInt(replicationText.getText());
				//transform the number from replication factor, to number of new copies
				factor-=1;
				if(factor<1){
					throw new Exception();
				}
			} catch (Exception x) {
				JOptionPane.showMessageDialog(this,
						"Fill inn the field correctly! Must be a integer greater than 1",
						"Missing data", JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (markedNode instanceof SubSystem) {				
				Exception error = tuner.tuneNonUniform(dimension, markedNode, Tuner.REPLICATE, (double)factor);
				if (error!=null) {// give feedback if errors occur
					reportError(error);
					return;
				}	
				
			} else if (markedNode instanceof Primitive) {
				//doesn't matter whether uniform or non-uniform is chosen
				Exception error = tuner.tuneNonUniform(dimension, markedNode, Tuner.REPLICATE, (double)factor);
				if (error!=null) {// give feedback if errors occur
					reportError(error);
					return;
				}
			}
			gainCalc.setCalcMethod(gainCalcPanel.getCalculationMethod());			
			gainCalc.calculateGainProcessing();
			gainCalc.calculateGainStorage();
			gainCalc.calculateGainConnectivity();
			frame.dispose();
		}else if (e.getSource() == gainCalcPanel.getOptionsButton()) {
			// open calcoptions panel in jdialog
			JOptionPane.showMessageDialog(this, "No options yet..", "Sorry",
					JOptionPane.INFORMATION_MESSAGE);
		}else if (e.getSource() == gainCalcPanel.getCancelButton()) {
			frame.dispose();
		}	
	}
	private void reportError(Exception exception){
		JTextArea text = new JTextArea();		
		text.append(exception.getMessage() + "\n");		
		JOptionPane.showMessageDialog(this, text,
				"Replication error", JOptionPane.ERROR_MESSAGE);
	
	}
}
