package gui.tune;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import computations.Tuner;

import datamodel.DataBaseHandler;
import datamodel.Primitive;
import datamodel.SubSystem;
import datamodel.System;

public class SubSystemTunePanel extends JPanel{

	private JTextField nameText;
	private JTextField maxUniformUpgradeText;
	private JTextField maxReplicationText;
	private JTextField replicationsLeftText;
	
	public SubSystemTunePanel() {
		super(new GridLayout(2,4));
		JLabel nameLabel=new JLabel("Name:");
		nameText=new JTextField();
		nameText.setEditable(false);
		JLabel maxUniformUpgradeLabel=new JLabel("Maximum uniform upgrade:");
		maxUniformUpgradeText=new JTextField();
		maxUniformUpgradeText.setEditable(false);
		JLabel maxReplicationLabel=new JLabel("Maximum replication:");
		maxReplicationText=new JTextField();
		maxReplicationText.setEditable(false);
		JLabel replicationsLeftLabel=new JLabel("Replications left:");
		replicationsLeftText=new JTextField();
		replicationsLeftText.setEditable(false);
		
		add(nameLabel);
		add(nameText);		
		add(maxUniformUpgradeLabel);
		add(maxUniformUpgradeText);
		add(maxReplicationLabel);
		add(maxReplicationText);
		add(replicationsLeftLabel);
		add(replicationsLeftText);
		setMaximumSize(new Dimension(800, 100));
		setMinimumSize(new Dimension(300, 50));
		setBorder(BorderFactory.createTitledBorder("Subsystem properties"));

	}
	public void setData(SubSystem subsys, int dimension){
		nameText.setText(subsys.getName());
		replicationsLeftText.setText(
				subsys.getMaxReplication()-subsys.getCurrentReplicationsCount()+"");
		maxReplicationText.setText(subsys.getMaxReplication()+"");
		double limit=getMinMaxUpgrade(subsys, dimension);
		maxUniformUpgradeText.setText(limit+"");			
	}
	private double getMinMaxUpgrade(SubSystem subsys, int dimension){
		double minmax=Double.MAX_VALUE;
		ArrayList<Primitive> devices=System.getAllPrimitives(subsys);
		switch (dimension) {
		case Tuner.PROCESSING:
			for (Primitive device : devices) {
				double temp=DataBaseHandler.getMaxUpgradeFactor(device.getProcessingSpeed(), Tuner.PROCESSING, device.getDeviceType());
				minmax=temp<minmax?temp:minmax;
			}
			break;
		case Tuner.STORAGE:
			for (Primitive device : devices) {
				double temp=DataBaseHandler.getMaxUpgradeFactor(device.getStorageCapacity(), Tuner.STORAGE, device.getDeviceType());
				minmax=temp<minmax?temp:minmax;
			}
			break;
		case Tuner.CONNECTIVITY:
			for (Primitive device : devices) {
				double temp=DataBaseHandler.getMaxUpgradeFactor(device.getConnectivity(), Tuner.CONNECTIVITY, device.getDeviceType());
				minmax=temp<minmax?temp:minmax;
			}
			break;
		default:
			break;
		}
		return minmax;
	}
	public void clearData(){
		nameText.setText("");
		maxUniformUpgradeText.setText("");
		maxReplicationText.setText("");
		replicationsLeftText.setText("");		
	}
}
