package gui.tune;

import gui.SAMeTreePanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.ReplicateScaleFilter;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

import computations.GainCalculator;
import computations.Tuner;
import datamodel.Component;
import datamodel.Primitive;
import datamodel.SubSystem;
import datamodel.System;

public class TunePanel extends JPanel implements TreeSelectionListener, ActionListener{
		
	private SAMeTreePanel treePanel;
	private SubSystemTunePanel subsysPanel;
	private PrimitiveTunePanel primitivePanel;
	private DimensionPanel dimensionPanel;
	private ModePanel modePanel;
	private TuneButtonPanel tuneButtonPanel;
	
	private JFrame frame;
	private System system;
	private GainCalculator gainCalc;
	private Tuner tuner;
	
	private Component markedNode;

	public TunePanel(JFrame frame, System system, GainCalculator gainCalc) {
		this.frame=frame;
		this.system=system;
		this.gainCalc=gainCalc;		
		this.tuner=system.getTuner();
		markedNode=system.getRootNode();
		
		treePanel=new SAMeTreePanel();
		treePanel.getTree().setModel(treePanel.makeTreeModel(system.getRootNode()));
		treePanel.getTree().addTreeSelectionListener(this);
		subsysPanel=new SubSystemTunePanel();
		primitivePanel=new PrimitiveTunePanel();
		dimensionPanel=new DimensionPanel();
		modePanel=new ModePanel();
		tuneButtonPanel=new TuneButtonPanel();
		
		dimensionPanel.getProcessingButton().addActionListener(this);
		dimensionPanel.getStorageButton().addActionListener(this);
		dimensionPanel.getConnectivityButton().addActionListener(this);
		
		tuneButtonPanel.getUpgradeButton().addActionListener(this);
		tuneButtonPanel.getReplicateButton().addActionListener(this);
		
		JPanel southPanel=new JPanel();
		southPanel.setLayout(new BoxLayout(southPanel,BoxLayout.X_AXIS));
		southPanel.add(Box.createRigidArea(new Dimension(10,0)));
		southPanel.add(dimensionPanel);
		southPanel.add(Box.createRigidArea(new Dimension(10,0)));
		southPanel.add(modePanel);
		southPanel.add(Box.createRigidArea(new Dimension(10,0)));
		southPanel.add(tuneButtonPanel);
		southPanel.add(Box.createRigidArea(new Dimension(10,0)));
				
		JPanel propertiesPanel=new JPanel();
		propertiesPanel.setLayout(new BoxLayout(propertiesPanel,BoxLayout.Y_AXIS));
		propertiesPanel.add(Box.createRigidArea(new Dimension(0,20)));
		propertiesPanel.add(subsysPanel);
		propertiesPanel.add(Box.createRigidArea(new Dimension(0,20)));
		propertiesPanel.add(primitivePanel);
		propertiesPanel.add(Box.createRigidArea(new Dimension(0,20)));
		propertiesPanel.add(southPanel);
		propertiesPanel.add(Box.createRigidArea(new Dimension(0,20)));
		JScrollPane scroll=new JScrollPane(propertiesPanel);
				
		JSplitPane splitPanel=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treePanel, scroll);		
		splitPanel.setDividerLocation(200);
		splitPanel.setPreferredSize(new Dimension(1000,600));
				
		setLayout(new BorderLayout());
		add(splitPanel, BorderLayout.CENTER);
		
		subsysPanel.setData((SubSystem)markedNode, Tuner.PROCESSING);
		primitivePanel.clearData();
		dimensionPanel.getReqText().setText(system.getRequirementProcessing()+"");
		dimensionPanel.getGainText().setText(gainCalc.getGainProcessing()+"");		
				
	}

	public void valueChanged(TreeSelectionEvent arg0) {
		//listen to tree and change data in propertypanels
		if (treePanel.getTree().getLastSelectedPathComponent() instanceof SubSystem) {
			markedNode = (SubSystem) treePanel.getTree().getLastSelectedPathComponent();
			//set data
			subsysPanel.setData((SubSystem)markedNode, dimensionPanel.getSelectedDimension());
			primitivePanel.clearData();
		}else if (treePanel.getTree().getLastSelectedPathComponent() instanceof Primitive) {
			markedNode = (Primitive) treePanel.getTree().getLastSelectedPathComponent();
			//set data
			primitivePanel.setData((Primitive)markedNode, dimensionPanel.getSelectedDimension());
			subsysPanel.clearData();
		}	
	}
	public void actionPerformed(ActionEvent e) {
		// listen to radiobuttons and buttons. change dimension and tune
		if(e.getSource()==dimensionPanel.getProcessingButton()){
			dimensionPanel.getReqText().setText(system.getRequirementProcessing()+"");
			dimensionPanel.getGainText().setText(gainCalc.getGainProcessing()+"");		
			if(markedNode instanceof SubSystem){
				subsysPanel.setData((SubSystem)markedNode, Tuner.PROCESSING);
			}else if(markedNode instanceof Primitive){
				primitivePanel.setData((Primitive)markedNode, Tuner.PROCESSING);
			}
		}else if(e.getSource()==dimensionPanel.getStorageButton()){
			dimensionPanel.getReqText().setText(system.getRequirementStorage()+"");
			dimensionPanel.getGainText().setText(gainCalc.getGainStorage()+"");
			if(markedNode instanceof SubSystem){
				subsysPanel.setData((SubSystem)markedNode, Tuner.STORAGE);
			}else if(markedNode instanceof Primitive){
				primitivePanel.setData((Primitive)markedNode, Tuner.STORAGE);
			}
		}else if(e.getSource()==dimensionPanel.getConnectivityButton()){
			dimensionPanel.getReqText().setText(system.getRequirementConnectivity()+"");
			dimensionPanel.getGainText().setText(gainCalc.getGainConnectivity()+"");	
			if(markedNode instanceof SubSystem){
				subsysPanel.setData((SubSystem)markedNode, Tuner.CONNECTIVITY);
			}else if(markedNode instanceof Primitive){
				primitivePanel.setData((Primitive)markedNode, Tuner.CONNECTIVITY);
			}
		}else if(e.getSource()==tuneButtonPanel.getUpgradeButton()){
			//show upgrade window
			if (markedNode instanceof SubSystem && modePanel.getMode() == Tuner.NONUNIFORM) {
				//nothing happens
				JOptionPane.showMessageDialog(this,
						"A subsystem cannot be upgraded non-uniformly.",
						"Change mode", JOptionPane.ERROR_MESSAGE);
				return;
			}
			JDialog win=new JDialog(frame, "Upgrade", true);
			win.getContentPane().add(new UpgradePanel(win, tuner, gainCalc, markedNode, dimensionPanel.getSelectedDimension(), modePanel.getMode()));
			win.pack();
			win.setVisible(true);
			if(dimensionPanel.getSelectedDimension()==Tuner.PROCESSING){
				dimensionPanel.getGainText().setText(gainCalc.getGainProcessing()+"");
			}else if(dimensionPanel.getSelectedDimension()==Tuner.STORAGE){
				dimensionPanel.getGainText().setText(gainCalc.getGainStorage()+"");
			}else if(dimensionPanel.getSelectedDimension()==Tuner.CONNECTIVITY){
				dimensionPanel.getGainText().setText(gainCalc.getGainConnectivity()+"");
			}
		}else if(e.getSource()==tuneButtonPanel.getReplicateButton()){
			//show replication window
			if(markedNode==system.getRootNode()){
				JOptionPane.showMessageDialog(this,
						"Cannot replicate the root!",
						"Error", JOptionPane.ERROR_MESSAGE);				
			}else{
				JDialog win=new JDialog(frame, "Upgrade", true);
				win.getContentPane().add(new ReplicationPanel(win, tuner, gainCalc, markedNode, dimensionPanel.getSelectedDimension()));
				win.pack();
				win.setVisible(true);
				if(dimensionPanel.getSelectedDimension()==Tuner.PROCESSING){
					dimensionPanel.getGainText().setText(gainCalc.getGainProcessing()+"");
				}else if(dimensionPanel.getSelectedDimension()==Tuner.STORAGE){
					dimensionPanel.getGainText().setText(gainCalc.getGainStorage()+"");
				}else if(dimensionPanel.getSelectedDimension()==Tuner.CONNECTIVITY){
					dimensionPanel.getGainText().setText(gainCalc.getGainConnectivity()+"");
				}
				treePanel.getTree().setModel(treePanel.makeTreeModel(system.getRootNode()));
			}
		}
		
	}
	
	

}
