package gui.tune;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import computations.GainCalculator;
import computations.Tuner;

import datamodel.Component;
import datamodel.Primitive;
import datamodel.SubSystem;
import datamodel.UpgradeLimitExceededException;

public class UpgradePanel extends JPanel implements ActionListener{

	private GainCalcPanel gainCalcPanel;
	
	private JTextField upgradeText;
	
	private Window frame;
	private Tuner tuner;
	private GainCalculator gainCalc;
	private Component markedNode;
	private int dimension;
	private int mode;	
	
	public UpgradePanel(Window frame, Tuner tuner, GainCalculator gainCalc, 
			Component markedNode, int dimension, int mode) {
		super(new BorderLayout());
		this.frame=frame;
		this.tuner=tuner;
		this.gainCalc=gainCalc;
		this.markedNode=markedNode;
		this.dimension=dimension;
		this.mode=mode;
				
		JLabel upgradeLabel=new JLabel("Upgrade factor:");
		upgradeText=new JTextField();
		if (markedNode instanceof Primitive) {
			switch (dimension) {
			case Tuner.PROCESSING:
				upgradeText.setText(((Primitive)markedNode).getUpgradeFactorProcessing()+"");
				break;
			case Tuner.STORAGE:
				upgradeText.setText(((Primitive)markedNode).getUpgradeFactorStorage()+"");
				break;
			case Tuner.CONNECTIVITY:
				upgradeText.setText(((Primitive)markedNode).getUpgradeFactorConnectivity()+"");
				break;
			default:
				break;
			}			
		}
		upgradeText.selectAll();
		
		JPanel textPanel=new JPanel(new GridLayout(0,2));
		textPanel.add(upgradeLabel);
		textPanel.add(upgradeText);
		
		gainCalcPanel=new GainCalcPanel();
		gainCalcPanel.getCalcButton().addActionListener(this);
		gainCalcPanel.getOptionsButton().addActionListener(this);
		gainCalcPanel.getCancelButton().addActionListener(this);
		
		add(textPanel, BorderLayout.NORTH);
		add(gainCalcPanel, BorderLayout.SOUTH);
		
	}
	
	public void actionPerformed(ActionEvent e) {
		// check if factor is less than original
		double factor = 1;
		if (e.getSource() == gainCalcPanel.getCalcButton()) {
			try {
				factor = Double.parseDouble(upgradeText.getText());
				if(factor<1){
					throw new Exception();
				}
			} catch (Exception x) {
				JOptionPane.showMessageDialog(this,
						"Fill inn the field correctly! (Must to be a positive number)",
						"Missing data", JOptionPane.ERROR_MESSAGE);
				return;
			}
			if (markedNode instanceof SubSystem) {
				if (mode == Tuner.UNIFORM) {
					ArrayList<UpgradeLimitExceededException> errorList = tuner
							.tuneUniform((SubSystem) markedNode, dimension,
									factor);
					boolean justErrors=true;
					while(errorList.remove(null))justErrors=false;//remove null objects
					if (errorList.size() > 0) {// give feedback if errors occur
						reportErrors(errorList);
						if(justErrors){
							return;
						}
					}					
				}
			} else if (markedNode instanceof Primitive) {
				//doesn't matter whether uniform or non-uniform is chosen
				Exception error = tuner
					.tuneNonUniform(dimension, (Primitive) markedNode, Tuner.UPGRADE, factor);
				if (error!=null) {// give feedback if errors occur
					reportError(error);
					return;
				}
			}
			gainCalc.setCalcMethod(gainCalcPanel.getCalculationMethod());
			switch (dimension) {
			case Tuner.PROCESSING:
				gainCalc.calculateGainProcessing();
				break;
			case Tuner.STORAGE:
				gainCalc.calculateGainStorage();
				break;
			case Tuner.CONNECTIVITY:
				gainCalc.calculateGainConnectivity();
				break;
			default:
				break;
			}			
			frame.dispose();
		}
		if (e.getSource() == gainCalcPanel.getOptionsButton()) {
			// open calcoptions panel in jdialog
			JOptionPane.showMessageDialog(this, "No options yet..", "Sorry",
					JOptionPane.INFORMATION_MESSAGE);
		}
		if (e.getSource() == gainCalcPanel.getCancelButton()) {
			frame.dispose();
		}

	}
	
	private void reportErrors(ArrayList<UpgradeLimitExceededException> errorList){
		JTextArea text = new JTextArea();
		for (UpgradeLimitExceededException exception : errorList) {
			text.append(exception.getMessage() + "\n");
		}
		JOptionPane.showMessageDialog(this, text,
				"Upgrade error", JOptionPane.ERROR_MESSAGE);
	
	}
	private void reportError(Exception exception){
		JTextArea text = new JTextArea();		
		text.append(exception.getMessage() + "\n");		
		JOptionPane.showMessageDialog(this, text,
				"Upgrade error", JOptionPane.ERROR_MESSAGE);
	
	}

}
