package simulator;

import java.util.ArrayList;

import datamodel.Primitive;
import desmoj.core.dist.RealDistExponential;
import desmoj.core.simulator.Model;
import desmoj.core.simulator.SimTime;

public class SimulatorModel extends Model {
	
	private ArrayList<Primitive> devices;

	private ArrayList<SAMResource> resources;

	private ArrayList<RealDistExponential> distributions;

	private int numOfResources;

	private double SIM_TRACE_PERIOD = 0;
	private double SIM_DEBUG_PERIOD = 0;

	private int NUM_OF_USERS = 0;

	//private int RUNTIME = 10000;//overridden in GainCalculator

	static String FILENAME;

	public SimulatorModel(Model owner, String modelName, boolean showInReport,
			boolean showInTrace, ArrayList<Primitive> dev) {
		super(owner, modelName, showInReport, showInTrace);		
		resources = new ArrayList<SAMResource>();
		distributions = new ArrayList<RealDistExponential>();
		//MyFileReader.readInputFromSP(FILENAME);
		devices=dev;
		numOfResources = devices.size();
	}

	public String description() {
		return "This is the SimulatorModel description. "
				+ "This model is a process oriented version with User objects that hold "
				+ "SAMResources. Simulation time is in seconds.";
	}

	public double getServiceDemand(int i) {
		return distributions.get(i).sample();
	}

	public ArrayList<SAMResource> getResources() {
		return resources;
	}

	public void doInitialSchedules() {
		for (int i = 1; i <= NUM_OF_USERS; i++) { // # of users in system at simulation
			// start
			Source aSource = new Source(this, "userArrival", true);
			aSource.schedule(new SimTime(0.0));
		}
	}

	public void init() {
		for (int i = 0; i < numOfResources; i++) {
			resources.add(new SAMResource(this, devices.get(i).getName(),
					devices.get(i).getReplication(), true, true));
		}
		RealDistExponential temp;
		for (int i = 0; i < numOfResources; i++) {
			temp = new RealDistExponential(this, devices.get(i).getName(),
					devices.get(i).getScaleSD(), true, false);
			temp.setNonNegative(true);
			distributions.add(temp);
		}
		//begin LD Effect/////
		/*
	    double thinktime=812.0;
	    double gc=(0.009*(Math.pow(NUM_OF_USERS, 1.810)))/100;
	    double x=NUM_OF_USERS/thinktime;
	    double GCfactor=gc/x;
	    temp = new RealDistExponential(this, devices.get(0).getName(),
				devices.get(0).getScaleSD()+GCfactor, true, false);
		temp.setNonNegative(true);
		distributions.set(0, temp);
		*/
		//end LDE/////////
	}

	public double getSIM_TRACE_PERIOD() {
		return SIM_TRACE_PERIOD;
	}

	public void setSIM_TRACE_PERIOD(double sim_trace_period) {
		SIM_TRACE_PERIOD = sim_trace_period;
	}

	public int getNUM_OF_USERS() {
		return NUM_OF_USERS;
	}

	public void setNUM_OF_USERS(int num_of_users) {
		NUM_OF_USERS = num_of_users;
	}
	
	public double getSIM_DEBUG_PERIOD() {
		return SIM_DEBUG_PERIOD;
	}

	public void setSIM_DEBUG_PERIOD(double sim_debug_period) {
		SIM_DEBUG_PERIOD = sim_debug_period;
	}

}